# Generated by Django 3.2.11 on 2022-04-14 14:41

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pharmacy", "0003_alter_rxrefill_unique_together"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalrx",
            name="medication",
        ),
        migrations.RemoveField(
            model_name="rx",
            name="medication",
        ),
        migrations.AddField(
            model_name="historicalrx",
            name="rx_identifier",
            field=models.CharField(default=uuid.uuid4, max_length=36),
        ),
        migrations.AddField(
            model_name="historicalrx",
            name="rx_name",
            field=models.CharField(default="study prescription", max_length=36),
        ),
        migrations.AddField(
            model_name="historicalrxrefill",
            name="refill_identifier",
            field=models.CharField(default=uuid.uuid4, max_length=36),
        ),
        migrations.AddField(
            model_name="rx",
            name="medications",
            field=models.ManyToManyField(to="edc_pharmacy.Medication"),
        ),
        migrations.AddField(
            model_name="rx",
            name="rx_identifier",
            field=models.CharField(default=uuid.uuid4, max_length=36),
        ),
        migrations.AddField(
            model_name="rx",
            name="rx_name",
            field=models.CharField(default="study prescription", max_length=36),
        ),
        migrations.AddField(
            model_name="rxrefill",
            name="refill_identifier",
            field=models.CharField(default=uuid.uuid4, max_length=36),
        ),
    ]
